#ifndef __TAJMAHAL_E__
#define __TAJMAHAL_E__

/* 
	MODULO PER LA GESTIONE DEL MODELLO DEL TAJMAHAL , QUESTO MODULO È  NON RIENTRANTE , 
	QUINDI IL COSTRUTTORE RESTITUISCE SEMPRE IL PUNTATORE ALLO STESSO OGGETTO
*/

#include "descriptor/descriptor.h"

/* costruttore , restituisce la radice della struttura tajmahal per le trasformazioni geometriche */
/* come input richiede il path della libreria corrente */
ITEM* new_tajmahal( char* path ) ;

/* ********************************************************************************************
IMPLEMENTAZIONE 
********************************************************************************************* */

#include "../common/utilities.e" /* read_dir_obj */

/* flag che indica se il tajmahal è stato inizializzato oppure no */
static int bool_inizialized = 0 ;

/* struttura non rientrante del tajmahal */
static ITEM brown,gray;
static ITEM piattaforma,torre1,torre2,torre3,torre4;
static ITEM facciata1,facciata2,facciata3,facciata4;
static ITEM *colonna, *cupolina, *portale, *finestra, *cup, *spigolo_back;
static ITEM spigolo1,spigolo2,spigolo3,spigolo4;
static ITEM cappella1,cappella2,cappella3,cappella4,piano, *cupola, *guglia;
/* radice dell' oggetto taymahal ra ritornare , serve per le trasformazioni geometriche */
static ITEM scene;

ITEM* new_tajmahal( char* path ){
	char *s_path , *path_texture_murato , *path_texture_entrata , *path_texture_finestra ;
	/* controllo se ho gia inizializzato l' oggetto */
	if ( bool_inizialized == 1 ) return &scene ;
	/* non ho ancora inizializzato il tajmahal */
	bool_inizialized = 1 ;
	/* trovo i path verso la sottodirectory s */
	s_path = concat_dir( path , "superfici" ) ;
	/* trovo il path per le texture */
	path_texture_murato = concat_dir( path , "texture/murato.hr" ) ;
	path_texture_entrata = concat_dir( path , "texture/entrata.hr" ) ;
	path_texture_finestra = concat_dir( path , "texture/finestra.hr" ) ;
	/*** ATTRIBUTE brown ***/
	brown = create_attribute("brown");
	set_color(brown,0.43,0.2,0.0,0.2,0.8);
	set_reflectivity(brown,0.17,15);

	/*** ATTRIBUTE gray ***/
	gray = create_attribute("gray");
	set_color(gray,1.0,0.866,0.855,0.1,0.8);
	  

	/*** Piattaforma ***/
	piattaforma = read_dir_obj("piattaforma.obj", s_path , "piattaforma");
	set_scale(piattaforma, 0.5, 0.5, 1);
	set_projection_texture(piattaforma, gray, path_texture_murato , TRUE, 2, 0.6, 0.8);
	set_scale(piattaforma, 1.0/0.5, 1.0/0.5, 1);


	/*** Torri ***/
	torre1 = read_dir_obj("torre.obj", s_path , "torre1");
	set_translate(torre1,0.3,0.3,0);
	
	torre2 = create_copy(torre1, "torre2");
	set_translate(torre2,-2.6,0,0);

	torre3 = create_copy(torre1, "torre3");
	set_translate(torre3,-2.6,-2.6,0);

	torre4 = create_copy(torre1, "torre4");
	set_translate(torre4,0,-2.6,0);

	colonna = get_surface( "superfici/pilastro.db" , path ) ;
	set_translate(*colonna, -1.0, -1.0, -0.1);
	
	set_projection_texture(*colonna, gray, path_texture_murato, TRUE, 2, 0.6, 0.8);
	set_translate(*colonna, 1.0, 1.0, 0.1);
	set_translate(*colonna,0.3,0.3,0);

	cupolina = get_surface( "superfici/cupolapiccolaS.db" , path ) ;
	set_translate(*cupolina, -1.0, -1.0, -0.5);
	set_projection_texture(*cupolina, gray, path_texture_murato, TRUE, 2, 0.6, 0.8);
	set_translate(*cupolina, 1.0, 1.0, 0.5);
	set_translate(*cupolina,0.3,0.3,0);
	
	set_attribute(torre1,gray);
	set_attribute(torre2,gray);
	set_attribute(torre3,gray);
	set_attribute(torre4,gray);

	/*** Facciate ***/
	facciata1 = read_dir_obj("facciata.obj", s_path , "facciata1");

	facciata2 = create_copy(facciata1, "facciata2");
	set_z_rotate(facciata2,90,1);
	
	facciata3 = create_copy(facciata1, "facciata3");
        set_z_rotate(facciata3,180,1);

	facciata4 = create_copy(facciata1, "facciata4");
        set_z_rotate(facciata4,-90,1);

		/*  Ingresso */
	portale = get_surface( "superfici/facciata_back.db" , path ) ;
	set_translate(*portale, 0.0, -0.7, 0.2);
	set_scale(*portale, 1.0/0.15, 1.0, 1.0/0.3);
	set_projection_texture(*portale, gray, path_texture_entrata, TRUE, 2, 0.6, 0.8);
	set_scale(*portale, 0.15, 1.0, 0.3);
	set_translate(*portale, 0.0, 0.7, -0.2);

		/* Finestre*/
	finestra = get_surface( "superfici/finestra_back.db" , path ) ;
	set_translate(*finestra, -0.375, -0.7, -0.15);
	set_scale(*finestra, 1.0/0.08, 1.0, 1.0/0.1);
	set_projection_texture(*finestra, gray, path_texture_finestra, TRUE, 2, 0.6, 0.8);
	set_scale(*finestra, 0.08, 1.0, 0.1);
	set_translate(*finestra, 0.375, 0.7, 0.15);		
	
	set_attribute(facciata1,gray);
	set_attribute(facciata2,gray);
	set_attribute(facciata3,gray);
	set_attribute(facciata4,gray);

	/*** Piano Superiore ***/
	piano = read_dir_obj("pianosuperiore.obj", s_path , "pianoSuperiore");
	set_attribute(piano,gray);

	/*** Cupola***/
	cupola = get_surface( "superfici/cupolaS.db" , path ) ;
	set_translate(*cupola, -0.45, -0.45, -0.8);
	set_projection_texture(*cupola, gray, path_texture_murato, TRUE, 2, 0.6, 0.8);
	set_translate(*cupola, 0.45, 0.45, 0.8);

	/*** Guglia ***/
	guglia = get_surface( "superfici/pennonecentrale2.db" , path ) ;
	set_attribute(*guglia,brown);

	/*** Cappelle ***/
	cappella1 = read_dir_obj("cappella.obj", s_path , "cappela1");
	
	cappella2 = create_copy(cappella1, "cappella2");
	set_translate(cappella2,-0.9,0,0);

	cappella3 = create_copy(cappella1, "cappella3");
	set_translate(cappella3,-0.9,-0.9,0);
	
	cappella4 = create_copy(cappella1, "cappella4");
	set_translate(cappella4,0,-0.9,0);

	cup = get_surface( "superfici/cupola.db" , path ) ;
	set_translate(*cup,  -0.45, -0.45, -0.8);
	set_projection_texture(*cup, gray, path_texture_murato, TRUE, 2, 0.6, 0.8);	
	set_translate(*cup, 0.45, 0.45, 0.8);

	set_attribute(cappella1,gray);
	set_attribute(cappella2,gray);
	set_attribute(cappella3,gray);
	set_attribute(cappella4,gray);

	/*** Spigoli ***/
	spigolo1 = read_dir_obj("spigolo.obj", s_path , "spigolo1");

	spigolo2 = create_copy(spigolo1, "spigolo2");
	set_z_rotate(spigolo2,90,1);

	spigolo4 = create_copy(spigolo1, "spigolo4");
        set_z_rotate(spigolo4,-90,1);
	
	spigolo3 = create_copy(spigolo4, "spigolo3");
        set_z_rotate(spigolo3,-90,1);

	spigolo_back = get_surface( "superfici/spigolo_back.db" , path ) ;
	set_translate(*spigolo_back, 0.7, -0.7, -0.35);
	set_z_rotate(*spigolo_back,-45,1);
	set_scale(*spigolo_back, 1.0/0.08, 1.0, 1.0/0.1);
	set_projection_texture(*spigolo_back, gray, path_texture_finestra, TRUE, 2, 0.6, 0.8);
	set_scale(*spigolo_back, 0.08, 1.0, 0.1);
	set_z_rotate(*spigolo_back,45,1);
	set_translate(*spigolo_back, -0.7, 0.7, 0.35);

	set_attribute(spigolo1,gray);
	set_attribute(spigolo2,gray);
	set_attribute(spigolo3,gray);
	set_attribute(spigolo4,gray);
	
	/* hierarchy initialization and construction */
	scene = create_list(piattaforma, "scene");
	scene = insert_in_list(scene, create_copy(*colonna, "colonna1"));
	set_translate(*colonna,-2.6,0,0);
	scene = insert_in_list(scene, create_copy(*colonna, "colonna2"));
	set_translate(*colonna,0,-2.6,0);
	scene = insert_in_list(scene, create_copy(*colonna, "colonna3"));
	set_translate(*colonna,2.6,0,0);
	scene = insert_in_list(scene, create_copy(*colonna, "colonna4"));
	scene = insert_in_list(scene, create_copy(*cupolina, "cupolina1"));
	set_translate(*cupolina,-2.6,0,0);
	scene = insert_in_list(scene, create_copy(*cupolina, "cupolina2"));
	set_translate(*cupolina,0,-2.6,0);
	scene = insert_in_list(scene, create_copy(*cupolina, "cupolina3"));
	set_translate(*cupolina,2.6,0,0);
	scene = insert_in_list(scene, create_copy(*cupolina, "cupolina4"));
		/* Ingresso*/
	
	scene = insert_in_list(scene, create_copy(*portale, "entrata1"));
	set_z_rotate(*portale,90,1);
	scene = insert_in_list(scene, create_copy(*portale, "entrata2"));
	set_z_rotate(*portale,90,1);
	scene = insert_in_list(scene, create_copy(*portale, "entrata3"));
	set_z_rotate(*portale,90,1);
	scene = insert_in_list(scene, create_copy(*portale, "entrata4"));
	
		/* finestre */
	scene = insert_in_list(scene, create_copy(*finestra, "back_sin_su1"));
	set_z_rotate(*finestra,90,1);
	scene = insert_in_list(scene, create_copy(*finestra, "back_sin_su2"));
	set_z_rotate(*finestra,90,1);
	scene = insert_in_list(scene, create_copy(*finestra, "back_sin_su3"));
	set_z_rotate(*finestra,90,1);
	scene = insert_in_list(scene, create_copy(*finestra, "back_sin_su4"));
	set_z_rotate(*finestra,90,1);
	
	set_translate(*finestra,0.0,0.0,-0.35);
	scene = insert_in_list(scene, create_copy(*finestra, "back_sin_giu1"));
	set_z_rotate(*finestra,90,1);
	scene = insert_in_list(scene, create_copy(*finestra, "back_sin_giu2"));
	set_z_rotate(*finestra,90,1);
	scene = insert_in_list(scene, create_copy(*finestra, "back_sin_giu3"));
	set_z_rotate(*finestra,90,1);
	scene = insert_in_list(scene, create_copy(*finestra, "back_sin_giu4"));
	set_z_rotate(*finestra,90,1);						
	
	set_translate(*finestra,-0.75,0.0,0.0);
	scene = insert_in_list(scene, create_copy(*finestra, "back_des_giu1"));
	set_z_rotate(*finestra,90,1);
	scene = insert_in_list(scene, create_copy(*finestra, "back_des_giu2"));
	set_z_rotate(*finestra,90,1);                            
	scene = insert_in_list(scene, create_copy(*finestra, "back_des_giu3"));
	set_z_rotate(*finestra,90,1);                                            
	scene = insert_in_list(scene, create_copy(*finestra, "back_des_giu4"));
	set_z_rotate(*finestra,90,1);

	set_translate(*finestra,0.0,0.0,0.35);
	scene = insert_in_list(scene, create_copy(*finestra, "back_des_su1"));
	set_z_rotate(*finestra,90,1);
	scene = insert_in_list(scene, create_copy(*finestra, "back_des_su2"));
	set_z_rotate(*finestra,90,1);
	scene = insert_in_list(scene, create_copy(*finestra, "back_des_su3"));
	set_z_rotate(*finestra,90,1);
	scene = insert_in_list(scene, create_copy(*finestra, "back_des_su4"));
	
		/* cup */
	scene = insert_in_list(scene, create_copy(*cup, "cup1"));
	set_translate(*cup,-0.9,0,0);
	scene = insert_in_list(scene, create_copy(*cup, "cup2"));
	set_translate(*cup,0,-0.9,0);
	scene = insert_in_list(scene, create_copy(*cup, "cup3"));
	set_translate(*cup,0.9,0,0);
	scene = insert_in_list(scene, create_copy(*cup, "cup4"));
	scene = insert_in_list(scene,torre1);
	scene = insert_in_list(scene,torre2);
	scene = insert_in_list(scene,torre3);
	scene = insert_in_list(scene,torre4);
	scene = insert_in_list(scene,facciata1);
	scene = insert_in_list(scene,facciata2);
	scene = insert_in_list(scene,facciata3);
	scene = insert_in_list(scene,facciata4);
	scene = insert_in_list(scene,piano);
	scene = insert_in_list(scene,*guglia);
	scene = insert_in_list(scene,*cupola);
	scene = insert_in_list(scene,cappella1);
	scene = insert_in_list(scene,cappella2);
	scene = insert_in_list(scene,cappella3);
	scene = insert_in_list(scene,cappella4);
		/* Spigoli */
	scene = insert_in_list(scene, create_copy(*spigolo_back, "spigolo_back_su1"));
	set_z_rotate(*spigolo_back,90,1);
	scene = insert_in_list(scene, create_copy(*spigolo_back, "spigolo_back_su2"));
	set_z_rotate(*spigolo_back,90,1);
	scene = insert_in_list(scene, create_copy(*spigolo_back, "spigolo_back_su3"));
	set_z_rotate(*spigolo_back,90,1);
	scene = insert_in_list(scene, create_copy(*spigolo_back, "spigolo_back_su4"));
	set_z_rotate(*spigolo_back,90,1);
	
	set_translate(*spigolo_back,0.0,0.0,-0.35);
	scene = insert_in_list(scene, create_copy(*spigolo_back, "spigolo_back_giu1"));
	set_z_rotate(*spigolo_back,90,1);
	scene = insert_in_list(scene, create_copy(*spigolo_back, "spigolo_back_giu2"));
	set_z_rotate(*spigolo_back,90,1);
	scene = insert_in_list(scene, create_copy(*spigolo_back, "spigolo_back_giu3"));
	set_z_rotate(*spigolo_back,90,1);
	scene = insert_in_list(scene, create_copy(*spigolo_back, "spigolo_back_giu4"));
							
	scene = insert_in_list(scene,spigolo1);
	scene = insert_in_list(scene,spigolo2);
	scene = insert_in_list(scene,spigolo3);
	scene = insert_in_list(scene,spigolo4);

	/* libero la stringa del path */
	free( s_path ) ;
	free( path_texture_murato ) ;
	free( path_texture_entrata ) ;
	free( path_texture_finestra ) ;
	return &scene ;
}

#endif

