#ifndef __UTILITIES_E__
#define __UTILITIES_E__

/*
	MODULO DI FUNZIONI DI UTILITA' GENERICA
*/

#include "descriptor/descriptor.h"

/* date due stringhe , queste vengono concatenate inserendo il delimitatore indicato */
/* la funzione restituisce una nuova stringa allocata in memoria oppure NULL in caso di errore */
char* concat( char *str1 , char *str2 , char* delimiter ) ;

/* funzione che concatena le directory per fare un percorso unico */
char* concat_dir( char *path1 , char *path2 ) ;

/* dato un nome di un file di superficie e un percorso , questa funzione costruisce l' oggetto ITEM
della superficie e lo restituisce */
ITEM* get_surface( char* name , char* path ) ;

/* date due stringhe non nulle , questa funzione concatena le due stringhe e costruisce un attributo 
con quel nome , che viene restituto */
ITEM* get_atribute( char *str1 , char* str2 ) ;

/* funzione simile alla read_obj della descriptor per leggere file obj di superfici . questa funzione sopperisce al fatto che la funzione read_obj della descriptor puo essere solo utilizzata quando lo script sta nella stessa directory del file obj e quindi con tutte le superfici , questa funzione infatti deve aver specificato il percorso per arrivare alla directory di superfici e quindi al file obj */
ITEM read_dir_obj(char *filename, char *path , char *name ) ;

/* **********************************************************************************************
IMPLEMENTAZIONE
*********************************************************************************************** */

#include <stdio.h>
#include <string.h>
#include <stdlib.h> /* malloc */

char* concat( char *str1 , char *str2 , char* delimiter ) {
	char *ris ;
	/* trovo la lunghezza della stringa da allocare */
	int total ;
	total = strlen( str1 ) ;
	total += strlen( str2 ) ;
	total += strlen( delimiter ) ;
	/* alloco la stringa in memoria , tenedo un posto per il carattere nullo di terminazione */
	ris = malloc( sizeof( char ) * total + 1 ) ;
	if ( ris == NULL ) {
		fprintf( stderr , "impossibile allocare ulteriore memoria\n" ) ;
		return NULL ;
	}
	/* copio la prima stringa */
	strcpy( ris , str1 ) ;
	/* copio il delimitatore in coda alla stringa */
	strcat( ris , delimiter ) ;
	/* copio la stringa finale */
	strcat( ris , str2 ) ;
	/* inserisco il terminatore di sctringa */
	ris[total] = '\0' ;
	/* ritorno la stringa formata */
	return ris ;
}

/* funzione che concatena le directory per fare un percorso unico */
char* concat_dir( char *path1 , char *path2 ) {
	/* controllo i valori in input */
	if (( path1 == NULL ) || ( path2 == NULL )) {
		fprintf( stderr , "valori di input non validi\n" ) ;
		exit( EXIT_FAILURE ) ;
	} else {
		char *ris ;
		int is_null ;
		/* controllo se path1 è nulla */
		is_null = strcmp( path1 , "" ) ;
		if ( is_null == 0 ) {
			ris = strdup( path2 ) ;
			/* la path è nulla , quindi ritorno un duplicato della path 2 */
			return ris ;
		}
		/* controllo se path2 è nulla */
		is_null = strcmp( path2 , "" ) ;
		if ( is_null == 0 ) {
			ris = strdup( path1 ) ;
			return ris ;
		}
		/* le due path sono integre , quindi le collego */
		ris = concat( path1 , path2 , "/" ) ;
		return ris ;
	}
}

ITEM* get_surface( char* name , char* path ) {
	/* controllo i valori in input */
	if (( name == NULL ) || ( path == NULL )) {
		fprintf( stderr , "valori di input non validi\n" ) ;
		exit( EXIT_FAILURE ) ;
	} else {
		char* file ;
		/* alloco memoria per la struttura */
		ITEM *new ;
		new = malloc( sizeof( ITEM )) ;
		if ( new == NULL ) {
			fprintf( stderr , "impossibile allocare ulteriore memoria\n" ) ;
			exit( EXIT_FAILURE ) ;	
		}
		/* creo il nome del fie da aprire , intanto controllo se la superficie è nella directory corrente*/
		int is_in_directory ;
		char* between ;
		is_in_directory = strcmp( path , "" ) ;
		switch( is_in_directory )
		{
			case 0 :
			{
				/* la directory è corrente , non devo inserire alcun separatore */
				between = "" ;
				break ;
			}
			default :
			{
				/* la directory è da un' altra parte , quindi devo inserire lo slash 
				come separatore */
				between = "/" ;
			}
		}
		file = concat( path , name , between ) ;
		/* carico il file sull' ITEM */
		*new = read_nurbs(file) ;
		/* dealloco il path appena trovato */
		free( file ) ;
		return new ;
	}
}

/* date due stringhe non nulle , questa funzione concatena le due stringhe e costruisce un attributo 
con quel nome , che viene restituto */
ITEM* get_attribute( char *str1 , char* str2 ) {
	/* controllo se le stringhe sono valide */
	if (( str1 == NULL ) || ( str2 == NULL)) {
		fprintf( stderr , "valori di input non validi\n" ) ;
		exit( EXIT_FAILURE ) ;
	} else {
		ITEM *new ;
		/* concateno le due stringhe */
		char* name ;
		name = concat( str1 , str2 , "_" ) ;
		/* alloco memoria per l' attributo */
		new = malloc( sizeof ( ITEM )) ;
		if ( new == NULL ) {
			fprintf( stderr , "impossibile allocare ulteriore memoria\n" ) ;
			exit( EXIT_FAILURE ) ;
		}
		/* costruisco un attributo cn questo nome */
		*new = create_attribute( name ) ;
		/* libero la memoria della stringa e restituisco il puntatore all'attributo */
		free( name ) ;
		return new ;
	}
}

/* funzione copiata dalla descriptor */
static void nextline_copied(FILE *fp, char *s) {
	int c, cnt = 0;

	/* skip white space */

	while ((c = getc (fp)) == ' ' ||
			   c == '\t' || c == '\n')
		;
	if (c == EOF)
		s[0] = '\0';
	else {
		s[cnt++] = (char) c;

		/* read the rest of the line */

		while ((c = getc (fp)) != EOF && c != '\n')
			s[cnt++] = (char) c;
		s[cnt] = '\0';
	}
}

/* il testo è praticamente la read_obj modificata */
ITEM read_dir_obj(char *filename, char *path , char *name ) {
  FILE *fp;
  ITEM it;
  char s[255],
  surf_fname[255];
  int  i,surf_no;
  /* ottengo l' indirizzo del filename */
  char* index ;
  index = concat_dir( path , filename ) ;
  /* cerco di aprire il file obj */
 
  if ((fp = fopen( index , "r")) == NULL) 
	  fprintf(stderr,"cannot open file");
  nextline_copied(fp, s);
  if ( sscanf(s, "FILENAME : %s", s ) != 1)
	  fprintf(stderr,"error reading objname");
  nextline_copied(fp, s);
  if (sscanf(s, "%d", &surf_no) != 1)
	  fprintf(stderr,"error reading number of surfaces");
  for (i=0; i<surf_no; i++) {
	  nextline_copied(fp, s);
	  if ( sscanf(s, "%s", surf_fname ) != 1) {
		  fprintf(stderr,"error reading surface name");
	  }
	  /* ho il nome della superficie , devo aprire l' indicizzo ,
	  intanto libero la memoria allocata precedentemente */
	  free( index ) ;
	  /* trovo l' indirizzo */
	  index = concat_dir( path , surf_fname ) ;
	  if(!i)
		  it = create_list(read_nurbs(index), name);
	  else
		  it = insert_in_list(it, read_nurbs(index));
  }
  
  /* libero l' ultima stringa allocata */
  free( index ) ;
  fclose(fp);
  return(it);
}

#endif

