#ifndef __BRACCIA_E__
#define __BRACCIA_E__

/* 
	MODULO PER LA GESTIONE DELLE SUPERFICI DELLE BRACCIA
*/

#include "descriptor/descriptor.h"

/* sottostrutture */
#include "spalle/spalle.e"
#include "braccia_superiori/braccia_superiori.e"
#include "braccia_inferiori/braccia_inferiori.e"
#include "mano/mano.e"

/* indico la struttura contentne tutti gli ITEM */
typedef struct {
	/* sottostrutture */
	spalle* spa ;
	braccia_sup* bra_sup ;
	braccia_inf* bra_inf ;
	mano* 	mano_sx ;
 	mano*	mano_dx ;
	/* oggetto ITEM che racchiude gli oggetti item delle superfici 
	delle sottostrutture per applicare le trasformazioni spaziali */
	ITEM list ;
} braccia ;

/* funzione che crea una nuova struttura e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
braccia* new_braccia( char *name , char *path , ITEM *spalle_a , ITEM *giunture_a , ITEM *parte_sup_a , ITEM *parte_inf_a ) ;

/* ********************************************************************************************
IMPLEMENTAZIONE 
********************************************************************************************* */

#include "../../common/utilities.e" /* concat */

#include <stdio.h> /* fprintf */
#include <stdlib.h> /* malloc , exit */
#include <string.h> /* strdup */

/* funzione che crea una nuova struttura e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
braccia* new_braccia( char *name , char *path , ITEM *spalle_a , ITEM *giunture_a , ITEM *parte_sup_a , ITEM *parte_inf_a ) {
	/* controllo i valori di input */
	if (( name == NULL ) || ( path == NULL ) || ( spalle_a == NULL ) ||
		( giunture_a == NULL ) || ( parte_sup_a == NULL ) || ( parte_inf_a == NULL )) {
		fprintf( stderr , "valori di input non validi\n" ) ;
		exit( EXIT_FAILURE ) ;
	} else {
		/* alloco memoria alla struttura */
		braccia* new ;
		new = malloc( sizeof ( braccia )) ;
		if ( new == NULL ){
			fprintf( stderr , "impossibile allocare ulteriore memoria\n" ) ;
			exit( EXIT_FAILURE ) ;
		}
		{
			char *path_spa , *path_bra_sup , *path_bra_inf , *path_man ;	
			/* ricavo gli indirizzi delle sottostrutture */
			path_spa = concat_dir( path , "spalle" ) ;
			path_bra_sup = concat_dir( path , "braccia_superiori" ) ;
			path_bra_inf = concat_dir( path , "braccia_inferiori" ) ;
			path_man = concat_dir( path , "mano" ) ;
			/* ricavo le sottostrutture */
			new->spa = new_spalle( "spalle" , path_spa , spalle_a ) ;
			new->bra_sup = new_braccia_sup( "braccia_superiori" , path_bra_sup , parte_sup_a ,giunture_a) ;
			new->bra_inf = new_braccia_inf( "braccia_inferiori" , path_bra_inf , parte_inf_a) ;
			new->mano_dx = new_mano( "mano_destra" , path_man , parte_inf_a) ;
			new->mano_sx = new_mano( "mano_sinistra" , path_man , parte_inf_a) ;
			/* libero la memoria delle stringhe */
			free( path_spa ) ;
			free( path_bra_sup ) ;
			free( path_bra_inf ) ;
			free( path_man ) ;
		}
		{
			/* trasformo le mani ( il modello è riferito alla mano destra ) */
			/* sinistra e destra sono riferiti dal punto di vista del mazinga */
			/* flag che indica che si utilizzano i gradi azichè i radianti */
			const int deg_flag = 1 ;
			set_z_rotate(new->mano_dx->list, -90.0, deg_flag );
			set_scale(new->mano_dx->list, 0.35, 0.35, 0.3);
			set_translate(new->mano_dx->list, 0.42, -1.20 , -3.05 ) ;
			set_z_rotate(new->mano_sx->list, -90.0, deg_flag );
			set_scale(new->mano_sx->list, 0.35, -0.35, 0.3);
			set_translate(new->mano_sx->list, 0.42, 1.10 , -3.05 ) ;
		}
		{
			/* costruisco il nome da dare alla lista */
			char* name_lista ;
			name_lista = concat( "lista" , name , "_" ) ;
			/* costruisco la lista di raggruppamento */
			new->list = create_list( new->spa->list , name_lista ) ;
			new->list = insert_in_list( new->list , new->bra_sup->list ) ;
			new->list = insert_in_list( new->list , new->bra_inf->list ) ;
			new->list = insert_in_list( new->list , new->mano_dx->list ) ;
			new->list = insert_in_list( new->list , new->mano_sx->list ) ;
		}
		/* restituisco l' oggetto */
		return new ;
	}
}

#endif 
