#ifndef __BRACCIA_SUPERIORI_E__
#define __BRACCIA_SUPERIORI_E__

/* 
	MODULO PER LA GESTIONE DELLE SUPERFICI DELLE BRACCIA SUPERIORI
*/

#include "descriptor/descriptor.h"

/* percorsi dei file di superficie a partire dalla directory della libreria corrente */
char* path_braccio_sup_dx = "s/braccio_sup_dx_s.db" ;
char* path_braccio_sup_sx = "s/braccio_sup_sx_s.db" ;
char* path_gomito_dx = "s/gomito_dx_s.db" ;
char* path_gomito_sx = "s/gomito_sx_s.db" ;


/* indico la struttura contentne tutti gli ITEM */
typedef struct {
	/* singoli oggetti ITEM per le singole superfici */
	ITEM 	*braccio_sup_dx ,
 			*braccio_sup_sx ,
			*gomito_dx ,
 			*gomito_sx ;
		/* oggetto ITEM che racchiude gli oggetti item delle superfici per applicare 
	le trasformazioni spaziali */
	ITEM list ;
} braccia_sup ;

/* funzione che crea una nuova struttura e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
braccia_sup* new_braccia_sup( char *name , char *path , ITEM *braccio_attr , ITEM *gomito_attr ) ;

/* ********************************************************************************************
IMPLEMENTAZIONE 
********************************************************************************************* */

#include "../../../common/utilities.e" /* concat */

#include <stdio.h> /* fprintf */
#include <stdlib.h> /* malloc , exit */
#include <string.h> /* strdup */

/* funzione che crea una nuova struttura e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
braccia_sup* new_braccia_sup( char *name , char *path , ITEM *braccio_attr , ITEM *gomito_attr ) {
	/* controllo i valori di input */
	if (( name == NULL ) || ( path == NULL ) || ( braccio_attr == NULL )|| ( gomito_attr == NULL )) {
		fprintf( stderr , "valori di input non validi\n" ) ;
		exit( EXIT_FAILURE ) ;
	} else {
		/* alloco memoria alla struttura */
		braccia_sup* new ;
		new = malloc( sizeof ( braccia_sup )) ;
		if ( new == NULL ){
			fprintf( stderr , "impossibile allocare ulteriore memoria\n" ) ;
			exit( EXIT_FAILURE ) ;
		}
		/* ricavo le superfici */
		new->braccio_sup_dx = get_surface( path_braccio_sup_dx , path ) ;
		new->braccio_sup_sx = get_surface( path_braccio_sup_sx , path ) ;
		new->gomito_dx = get_surface( path_gomito_dx , path ) ;
		new->gomito_sx = get_surface( path_gomito_sx , path ) ;
		/* collego le superfici agli attributi */
		set_attribute( *(new->braccio_sup_dx), *braccio_attr);
		set_attribute( *(new->braccio_sup_sx), *braccio_attr);
		set_attribute( *(new->gomito_dx), *gomito_attr);
		set_attribute( *(new->gomito_sx), *gomito_attr);
		{
			/* costruisco il nome da dare alla lista */
			char* name_lista ;
			name_lista = concat( "lista" , name , "_" ) ;
			/* costruisco la lista di raggruppamento */
			new->list = create_list( *(new->braccio_sup_dx), name_lista ) ;
			new->list = insert_in_list( new->list , *(new->braccio_sup_sx) ) ;
			new->list = insert_in_list( new->list , *(new->gomito_dx) ) ;
			new->list = insert_in_list( new->list , *(new->gomito_sx) ) ;
		}
		/* restituisco l' oggetto */
		return new ;
	}
}

#endif 

