#ifndef __MANO_E__
#define __MANO_E__

/* 
	MODULO PER LA GESTIONE DELLE SUPERFICI DELLA MANO
*/

#include "descriptor/descriptor.h"
#include "dito.e"

char* path_palmo = "s/palmo_s.db" ;
char* path_polso_pollice = "s/polso_pollice_s.db" ;
char* path_polso = "s/polso_s.db" ;
char* path_attacco_pollice = "s/attacco_pollice_s.db" ;
char* path_falange_pollice_up = "s/falange_pollice_up_s.db" ;
char* path_falange_pollice_down = "s/falange_pollice_down_s.db" ;
char* path_raccordo_pollice = "s/raccordo_pollice_s.db" ;
char* path_unghia_pollice = "s/unghia_pollice_s.db" ;

/* indico la struttura contentne tutti gli ITEM */
typedef struct {
	/* singoli oggetti ITEM per le singole superfici */
	ITEM 	*palmo , 
 			*polso_pollice ,
			*polso ,
   			*attacco_pollice ,
	  		*falange_pollice_up , 
	 		*falange_pollice_down , 
  			*raccordo_pollice , 
  			*unghia_pollice ;
	/* struttura delle dita */
	dito *indice , *medio , *anulare , *mignolo ;
		/* oggetto ITEM che racchiude gli oggetti item delle superfici per applicare 
 le trasformazioni spaziali */
 ITEM list ;
} mano ;

/* funzione che crea una nuova struttura e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
mano* new_mano( char *name , char *path , ITEM *attr ) ;

/* ********************************************************************************************
IMPLEMENTAZIONE 
********************************************************************************************* */

#include "../../../common/utilities.e" /* concat */

#include <stdio.h> /* fprintf */
#include <stdlib.h> /* malloc , exit */
#include <string.h> /* strdup */

/* funzione che crea una nuova struttura e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
mano* new_mano( char *name , char *path , ITEM *attr ) {
	/* controllo i valori di input */
	if (( name == NULL ) || ( path == NULL ) ||( attr == NULL )) {
		fprintf( stderr , "valori di input non validi\n" ) ;
		exit( EXIT_FAILURE ) ;
	} else {
		/* alloco memoria alla struttura */
		mano* new ;
		new = malloc( sizeof ( mano )) ;
		if ( new == NULL ){
			fprintf( stderr , "impossibile allocare ulteriore memoria\n" ) ;
			exit( EXIT_FAILURE ) ;
		}
		/* ricavo le superfici e assegno gli attributi */
		new->palmo = get_surface( path_palmo , path ) ;
		new->polso_pollice = get_surface( path_polso_pollice , path ) ;
		new->polso = get_surface( path_polso , path ) ;
		new->attacco_pollice = get_surface( path_attacco_pollice , path ) ;
		new->falange_pollice_up = get_surface( path_falange_pollice_up , path ) ;
		new->falange_pollice_down = get_surface( path_falange_pollice_down , path ) ;
		new->raccordo_pollice = get_surface( path_raccordo_pollice , path ) ;
		new->unghia_pollice = get_surface( path_unghia_pollice , path ) ;
		
		set_attribute( *(new->palmo), *attr);
		set_attribute( *(new->polso_pollice), *attr);
		set_attribute( *(new->polso), *attr);
		set_attribute( *(new->attacco_pollice), *attr);
		set_attribute( *(new->falange_pollice_up), *attr);
		set_attribute( *(new->falange_pollice_down), *attr);
		set_attribute( *(new->raccordo_pollice), *attr);
		set_attribute( *(new->unghia_pollice), *attr);
		{
			/* ricavo i nomi delle dita */
			char *name_indice , *name_medio , *name_anulare , *name_mignolo ;
			name_indice = concat( name , "indice" , "_" ) ;
			name_medio = concat( name , "medio" , "_" ) ;
			name_anulare = concat( name , "anulare" , "_" ) ;
			name_mignolo = concat( name , "mignolo" , "_" ) ;
			/* ottengo gli oggetti , che sono tutti nella cartella di mano */
			new->indice = new_dito( name_indice , path , attr ) ;
			new->medio = new_dito( name_medio , path , attr ) ;
			new->anulare = new_dito( name_anulare , path , attr ) ;
			new->mignolo = new_dito( name_mignolo , path , attr ) ;
			/* libero la memoria dei nomi */
			free( name_indice ) ; free( name_medio ) ; 
			free( name_anulare ) ; free( name_mignolo ) ; 
		}
		{
			/* traslo le dita , sono tutte posiZionate sull' indice */
			set_translate(new->medio->list, 0.0, -0.3 , 0.0 ) ;
			set_translate(new->anulare->list, 0.0, -0.6 , 0.0 ) ;
			set_translate(new->mignolo->list, 0.0, -0.9 , 0.0 ) ;
		}
		{
			/* costruisco il nome da dare alla lista */
			char* name_lista ;
			name_lista = concat( "lista" , name , "_" ) ;
			/* costruisco la lista di raggruppamento */
			new->list = create_list( *(new->palmo), name_lista ) ;
			new->list = insert_in_list( new->list , *(new->polso_pollice) ) ;
			new->list = insert_in_list( new->list , *(new->polso) ) ;
			new->list = insert_in_list( new->list , *(new->attacco_pollice) ) ;
			new->list = insert_in_list( new->list , *(new->falange_pollice_up) ) ;
			new->list = insert_in_list( new->list , *(new->falange_pollice_down) ) ;
			new->list = insert_in_list( new->list , *(new->raccordo_pollice) ) ;
			new->list = insert_in_list( new->list , *(new->unghia_pollice) ) ;
			new->list = insert_in_list( new->list , new->indice->list ) ;
			new->list = insert_in_list( new->list , new->medio->list ) ;
			new->list = insert_in_list( new->list , new->anulare->list ) ;
			new->list = insert_in_list( new->list , new->mignolo->list ) ;
		}
		/* restituisco l' oggetto */
		return new ;
	}
}

#endif 



