#ifndef __GAMBE_INFERIORI_E__
#define __GAMBE_INFERIORI_E__

/* 
	MODULO PER LA GESTIONE DELLE SUPERFICI DELLE GAMBE INFERIORI
*/

#include "descriptor/descriptor.h"

/* percorsi dei file di superficie a partire dalla directory della libreria corrente */
char* path_gamba_inf_dx = "s/gamba_inf_dx_s.db" ;
char* path_gamba_inf_sx = "s/gamba_inf_sx_s.db" ;
char* path_caviglia_dx = "s/caviglia_dx_s.db" ;
char* path_caviglia_sx = "s/caviglia_sx_s.db" ;

/* indico la struttura contentne tutti gli ITEM */
typedef struct {
	/* singoli oggetti ITEM per le singole superfici */
	ITEM 	*gamba_dx ,
 			*gamba_sx ,
			*caviglia_dx ,
   			*caviglia_sx ;
		/* oggetto ITEM che racchiude gli oggetti item delle superfici per applicare 
	le trasformazioni spaziali */
	ITEM list ;
} gambe_inf ;

/* funzione che crea una nuova struttura e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
gambe_inf* new_gambe_inf( char *name , char *path , ITEM *gamba_attr , ITEM *caviglia_attr ) ;

/* ********************************************************************************************
IMPLEMENTAZIONE 
********************************************************************************************* */

#include "../../../common/utilities.e" /* concat */

#include <stdio.h> /* fprintf */
#include <stdlib.h> /* malloc , exit */
#include <string.h> /* strdup */

/* funzione che crea una nuova struttura e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
gambe_inf* new_gambe_inf( char *name , char *path , ITEM *gamba_attr , ITEM *caviglia_attr ) {
	/* controllo i valori di input */
	if (( name == NULL )||( path == NULL )||( gamba_attr == NULL )||( caviglia_attr == NULL )) {
		fprintf( stderr , "valori di input non validi\n" ) ;
		exit( EXIT_FAILURE ) ;
	} else {
		/* alloco memoria alla struttura */
		gambe_inf* new ;
		new = malloc( sizeof ( gambe_inf )) ;
		if ( new == NULL ){
			fprintf( stderr , "impossibile allocare ulteriore memoria\n" ) ;
			exit( EXIT_FAILURE ) ;
		}
		/* ricavo le superfici */
		new->gamba_dx = get_surface( path_gamba_inf_dx , path ) ;
		new->gamba_sx = get_surface( path_gamba_inf_sx , path ) ;
		new->caviglia_dx = get_surface( path_caviglia_dx , path ) ;
		new->caviglia_sx = get_surface( path_caviglia_sx , path ) ;
		/* collego le superfici agli attributi */
		set_attribute( *(new->gamba_dx), *gamba_attr);
		set_attribute( *(new->gamba_sx), *gamba_attr);
		set_attribute( *(new->caviglia_dx), *caviglia_attr);
		set_attribute( *(new->caviglia_sx), *caviglia_attr);
		{
			/* costruisco il nome da dare alla lista */
			char* name_lista ;
			name_lista = concat( "lista" , name , "_" ) ;
			/* costruisco la lista di raggruppamento */
			new->list = create_list( *(new->gamba_dx), name_lista ) ;
			new->list = insert_in_list( new->list , *(new->gamba_sx) ) ;
			new->list = insert_in_list( new->list , *(new->caviglia_dx) ) ;
			new->list = insert_in_list( new->list , *(new->caviglia_sx) ) ;
		}
		/* restituisco l' oggetto */
		return new ;
	}
}

#endif 

