#ifndef __PIEDI_E__
#define __PIEDI_E__

/* 
	MODULO PER LA GESTIONE DELLE SUPERFICI DEI PIEDI
*/

#include "descriptor/descriptor.h"

/* percorsi dei file di superficie a partire dalla directory della libreria corrente */
char* path_pianta_dx = "s/pianta_dx_s.db" ;
char* path_pianta_sx = "s/pianta_sx_s.db" ;
char* path_superficie_lat_dx = "s/superficie_lat_dx_s.db" ;
char* path_superficie_lat_sx = "s/superficie_lat_sx_s.db" ;
char* path_suola_dx = "s/suola_dx_s.db" ;
char* path_suola_sx = "s/suola_sx_s.db" ;

/* indico la struttura contentne tutti gli ITEM */
typedef struct {
	/* singoli oggetti ITEM per le singole superfici */
	ITEM 	*pianta_dx ,
 			*pianta_sx ,
			*superficie_lat_dx ,
 			*superficie_lat_sx ,
			*suola_dx ,
 			*suola_sx ;
		/* oggetto ITEM che racchiude gli oggetti item delle superfici per applicare 
	le trasformazioni spaziali */
	ITEM list ;
} piedi ;

/* funzione che crea una nuova struttura e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
piedi* new_piedi( char *name , char *path , ITEM *piedi_attr , ITEM *suola_attr ) ;

/* ********************************************************************************************
IMPLEMENTAZIONE 
********************************************************************************************* */

#include "../../../common/utilities.e" /* concat */

#include <stdio.h> /* fprintf */
#include <stdlib.h> /* malloc , exit */
#include <string.h> /* strdup */

/* funzione che crea una nuova struttura e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
piedi* new_piedi( char *name , char *path , ITEM *piedi_attr , ITEM *suola_attr ) {
	/* controllo i valori di input */
if (( name == NULL )||( path == NULL )||( piedi_attr == NULL )||( suola_attr == NULL )) {
		fprintf( stderr , "valori di input non validi\n" ) ;
		exit( EXIT_FAILURE ) ;
	} else {
		/* alloco memoria alla struttura */
		piedi* new ;
		new = malloc( sizeof ( piedi )) ;
		if ( new == NULL ){
			fprintf( stderr , "impossibile allocare ulteriore memoria\n" ) ;
			exit( EXIT_FAILURE ) ;
		}
		/* ricavo le superfici */
		new->pianta_dx = get_surface( path_pianta_dx , path ) ;
		new->pianta_sx = get_surface( path_pianta_sx , path ) ;
		new->superficie_lat_dx = get_surface( path_superficie_lat_dx , path ) ;
		new->superficie_lat_sx = get_surface( path_superficie_lat_sx , path ) ;
		new->suola_dx = get_surface( path_suola_dx , path ) ;
		new->suola_sx = get_surface( path_suola_sx , path ) ;
		/* collego le superfici agli attributi */
		set_attribute( *(new->pianta_dx), *suola_attr);
		set_attribute( *(new->pianta_sx), *suola_attr);
		set_attribute( *(new->superficie_lat_dx), *piedi_attr);
		set_attribute( *(new->superficie_lat_sx), *piedi_attr);
		set_attribute( *(new->suola_dx), *suola_attr);
		set_attribute( *(new->suola_sx), *suola_attr);		
		{
			/* costruisco il nome da dare alla lista */
			char* name_lista ;
			name_lista = concat( "lista" , name , "_" ) ;
			/* costruisco la lista di raggruppamento */
			new->list = create_list( *(new->superficie_lat_dx), name_lista ) ;
			new->list = insert_in_list( new->list , *(new->superficie_lat_sx) ) ;
			new->list = insert_in_list( new->list , *(new->suola_dx) ) ;
			new->list = insert_in_list( new->list , *(new->suola_sx) ) ;
			new->list = insert_in_list( new->list , *(new->pianta_dx) ) ;
			new->list = insert_in_list( new->list , *(new->pianta_sx) ) ;
			
		}
		/* restituisco l' oggetto */
		return new ;
	}
}

#endif 

