#ifndef __AERATORE_CENTRALE_E__
#define __AERATORE_CENTRALE_E__

/* 
	MODULO PER LA GESTIONE DELLE SUPERFICI DELL' AERATORE CENTRALE
*/

#include "descriptor/descriptor.h"

/* percorsi dei file di superficie a partire dalla directory della libreria corrente */
char* path_fascia_centrale_dx = "s/fascia_centrale_dx_s.db" ;
char* path_fascia_centrale_sx = "s/fascia_centrale_sx_s.db" ;
char* path_fascia_down = "s/fascia_down_s.db" ;
char* path_fascia_up = "s/fascia_up_s.db" ;
char* path_fascia_sx = "s/fascia_sx_s.db" ;
char* path_fascia_dx = "s/fascia_dx_s.db" ;
char* path_pareti_aeratore_centro = "s/pareti_aeratore_centro_s.db" ;
char* path_pareti_aeratore_dx = "s/pareti_aeratore_dx_s.db" ;
char* path_pareti_aeratore_sx = "s/pareti_aeratore_sx_s.db" ;
char* path_sfondo_aeratori = "s/sfondo_aeratori_s.db" ;

/* indico la struttura contentne tutti gli ITEM */
typedef struct {
	/* singoli oggetti ITEM per le singole superfici */
	ITEM 	*fascia_centrale_dx , 
			*fascia_centrale_sx , 
			*fascia_down , 
			*fascia_up , 
			*fascia_sx , 
			*fascia_dx ,
			*pareti_aeratore_centro , 
			*pareti_aeratore_dx , 
			*pareti_aeratore_sx , 
			*sfondo_aeratori ;
	/* oggetto ITEM che racchiude gli oggetti item delle superfici per applicare 
	le trasformazioni spaziali */
	ITEM list ;
} aeratore_centrale ;

/* funzione che crea una nuova struttura aeratore e la restituisce , bisogna inserire il nome , 
 il path che va dal programma effettivo alla libreria e gli attributi utilizzati */
aeratore_centrale* new_aeratore_centrale( char* name , char *path , ITEM *sfondo_attr , ITEM *aeratore_attr ) ;

/* ********************************************************************************************
IMPLEMENTAZIONE 
********************************************************************************************* */

#include "../../../common/utilities.e" /* concat */

#include <stdio.h> /* fprintf */
#include <stdlib.h> /* malloc , exit */
#include <string.h> /* strdup */

/* funzione che crea una nuova struttura aeratore e la restituisce , bisogna inserire il nome , 
 il path che va dal programma effettivo alla libreria e gli attributi utilizzati */
aeratore_centrale* new_aeratore_centrale( char* name , char *path , ITEM *sfondo_attr , ITEM *aeratore_attr ) {
	/* controllo i valori di input */
	if (( name == NULL ) || ( path == NULL ) || ( sfondo_attr == NULL ) || ( aeratore_attr == NULL )) {
		fprintf( stderr , "valori di input non validi\n" ) ;
		exit( EXIT_FAILURE ) ;
	} else {
		/* alloco memoria alla struttura */
		aeratore_centrale* new ;
		new = malloc( sizeof ( aeratore_centrale )) ;
		if ( new == NULL ){
			fprintf( stderr , "impossibile allocare ulteriore memoria\n" ) ;
			exit( EXIT_FAILURE ) ;
		}
		/* ricavo le superfici */
		new->fascia_centrale_dx = get_surface( path_fascia_centrale_dx , path ) ;
		new->fascia_centrale_sx = get_surface( path_fascia_centrale_sx , path ) ;
		new->fascia_down = get_surface( path_fascia_down , path ) ;
		new->fascia_up = get_surface( path_fascia_up , path ) ;
		new->fascia_sx = get_surface( path_fascia_sx , path ) ;
		new->fascia_dx = get_surface( path_fascia_dx , path ) ;
		new->pareti_aeratore_centro = get_surface( path_pareti_aeratore_centro , path ) ;
		new->pareti_aeratore_dx = get_surface( path_pareti_aeratore_dx , path ) ;
		new->pareti_aeratore_sx = get_surface( path_pareti_aeratore_sx , path ) ;
		new->sfondo_aeratori = get_surface( path_sfondo_aeratori , path ) ;
		/* setto gli atributi */
		set_attribute( *(new->fascia_centrale_dx) , *aeratore_attr);
		set_attribute( *(new->fascia_centrale_sx), *aeratore_attr);
		set_attribute( *(new->fascia_down), *aeratore_attr);
		set_attribute( *(new->fascia_up), *aeratore_attr);
		set_attribute( *(new->fascia_sx), *aeratore_attr);
		set_attribute( *(new->fascia_dx), *aeratore_attr);
		set_attribute( *(new->pareti_aeratore_centro), *aeratore_attr);
		set_attribute( *(new->pareti_aeratore_dx), *aeratore_attr);
		set_attribute( *(new->pareti_aeratore_sx), *aeratore_attr);
		set_attribute( *(new->sfondo_aeratori), *sfondo_attr);
		{
			/* costruisco il nome da dare alla lista */
			char* name_lista ;
			name_lista = concat( "lista" , name , "_" ) ;
			/* costruisco la lista di raggruppamento */
			new->list = create_list( *(new->fascia_centrale_dx ), name_lista ) ;
			new->list = insert_in_list( new->list , *(new->fascia_centrale_sx) ) ;
			new->list = insert_in_list( new->list , *(new->fascia_down) ) ;
			new->list = insert_in_list( new->list , *(new->fascia_up) ) ;
			new->list = insert_in_list( new->list , *(new->fascia_sx) ) ;
			new->list = insert_in_list( new->list , *(new->fascia_dx) ) ;
			new->list = insert_in_list( new->list , *(new->pareti_aeratore_centro) ) ;
			new->list = insert_in_list( new->list , *(new->pareti_aeratore_dx) ) ;
			new->list = insert_in_list( new->list , *(new->pareti_aeratore_sx) ) ;
			new->list = insert_in_list( new->list , *(new->sfondo_aeratori) ) ;
			free( name_lista ) ;
		}
		/* restituisco l' oggetto */
		return new ;
	}
}

#endif 
