#ifndef __BOCCA_E__
#define __BOCCA_E__

/* 
	MODULO PER LA GESTIONE DELLE SUPERFICI DELLA BOCCA
*/

#include "descriptor/descriptor.h"

/* percorsi dei file di superficie a partire dalla directory della libreria corrente */
char* path_bocca_orecchio_dx_s = "s/bocca_orecchio_dx_s.db" ;
char* path_bocca_orecchio_sx_s = "s/bocca_orecchio_sx_s.db" ;
char* path_labbro_totale_s = "s/labbro_totale_s.db" ;
char* path_mento_totale_s = "s/mento_totale_s.db" ;

/* indico la struttura contentne tutti gli ITEM */
typedef struct {
	/* singoli oggetti ITEM per le singole superfici */
	ITEM 	*bocca_orecchio_dx , 
 			*path_bocca_orecchio_sx , 
 			*path_labbro_totale , 
 			*path_mento_totale ;
	/* oggetto ITEM che racchiude gli oggetti item delle superfici per applicare 
	le trasformazioni spaziali */
	ITEM list ;
} bocca ;

/* funzione che crea una nuova struttura bocca e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
bocca* new_bocca( char *name , char *path , ITEM *attr ) ;

/* ********************************************************************************************
IMPLEMENTAZIONE 
********************************************************************************************* */

#include "../../../common/utilities.e" /* concat */

#include <stdio.h> /* fprintf */
#include <stdlib.h> /* malloc , exit */
#include <string.h> /* strdup */

/* funzione che crea una nuova struttura bocca e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
bocca* new_bocca( char *name , char *path , ITEM *attr ) {
	/* controllo i valori di input */
	if (( name == NULL ) || ( path == NULL ) || ( attr == NULL )) {
		fprintf( stderr , "valori di input non validi\n" ) ;
		exit( EXIT_FAILURE ) ;
	} else {
		/* alloco memoria alla struttura */
		bocca* new ;
		new = malloc( sizeof ( bocca )) ;
		if ( new == NULL ){
			fprintf( stderr , "impossibile allocare ulteriore memoria\n" ) ;
			exit( EXIT_FAILURE ) ;
		}
		/* ricavo le superfici */
		new->bocca_orecchio_dx = get_surface( path_bocca_orecchio_dx_s , path ) ;
		new->path_bocca_orecchio_sx = get_surface( path_bocca_orecchio_sx_s , path ) ;
		new->path_labbro_totale = get_surface( path_labbro_totale_s , path ) ;
		new->path_mento_totale = get_surface( path_mento_totale_s , path ) ;
		/* imposto gli attributi */
		set_attribute( *(new->bocca_orecchio_dx) , *attr);
		set_attribute( *(new->path_bocca_orecchio_sx), *attr);
		set_attribute( *(new->path_labbro_totale), *attr);
		set_attribute( *(new->path_mento_totale), *attr);
		{
			/* costruisco il nome da dare alla lista */
			char* name_lista ;
			name_lista = concat( "lista" , name , "_" ) ;
			/* costruisco la lista di raggruppamento */
			new->list = create_list( *(new->bocca_orecchio_dx ), name_lista ) ;
			new->list = insert_in_list( new->list , *(new->path_bocca_orecchio_sx) ) ;
			new->list = insert_in_list( new->list , *(new->path_labbro_totale) ) ;
			new->list = insert_in_list( new->list , *(new->path_mento_totale) ) ;
			free( name_lista ) ;
		}
		/* restituisco l' oggetto */
		return new ;
	}
}

#endif 
