#ifndef __CORNA_ORECCHIE_E__
#define __CORNA_ORECCHIE_E__

/* 
	MODULO PER LA GESTIONE DELLE SUPERFICI DI CORNA E ORECCHIE
*/

#include "descriptor/descriptor.h"

/* percorsi dei file di superficie a partire dalla directory della libreria corrente */
char* path_orecchio_dx = "s/orecchio_dx_s.db" ;
char* path_orecchio_sx = "s/orecchio_sx_s.db" ;
char* path_corno_dx = "s/corno_dx_s.db" ;
char* path_corno_sx = "s/corno_sx_s.db" ;

/* indico la struttura contentne tutti gli ITEM */
typedef struct {
	/* singoli oggetti ITEM per le singole superfici */
	ITEM 	*orecchio_dx , 
 			*orecchio_sx , 
			*corno_dx , 
 			*corno_sx ;
		/* oggetto ITEM che racchiude gli oggetti item delle superfici per applicare 
	le trasformazioni spaziali */
	ITEM list ;
} corna_orecchie ;

/* funzione che crea una nuova struttura e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
corna_orecchie* new_corna_orecchie( char *name , char *path , ITEM *corno_attr , ITEM *orecchio_attr ) ;

/* ********************************************************************************************
IMPLEMENTAZIONE 
********************************************************************************************* */

#include "../../../common/utilities.e" /* concat */

#include <stdio.h> /* fprintf */
#include <stdlib.h> /* malloc , exit */
#include <string.h> /* strdup */

/* funzione che crea una nuova struttura e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
corna_orecchie* new_corna_orecchie( char *name , char *path , ITEM *corno_attr , ITEM *orecchio_attr ) {
	/* controllo i valori di input */
	if (( name == NULL ) || ( path == NULL ) || ( corno_attr == NULL ) || ( orecchio_attr == NULL )) {
		fprintf( stderr , "valori di input non validi\n" ) ;
		exit( EXIT_FAILURE ) ;
	} else {
		/* alloco memoria alla struttura */
		corna_orecchie* new ;
		new = malloc( sizeof ( corna_orecchie )) ;
		if ( new == NULL ){
			fprintf( stderr , "impossibile allocare ulteriore memoria\n" ) ;
			exit( EXIT_FAILURE ) ;
		}
		/* ricavo le superfici */
		new->orecchio_dx = get_surface( path_orecchio_dx , path ) ;
		new->orecchio_sx = get_surface( path_orecchio_sx , path ) ;
		new->corno_dx = get_surface( path_corno_dx , path ) ;
		new->corno_sx = get_surface( path_corno_sx , path ) ;
		/* imposto gli attributi */
		set_attribute( *(new->orecchio_dx) , *orecchio_attr);
		set_attribute( *(new->orecchio_sx) , *orecchio_attr);
		set_attribute( *(new->corno_dx), *corno_attr);
		set_attribute( *(new->corno_sx), *corno_attr);
		{
			/* costruisco il nome da dare alla lista */
			char* name_lista ;
			name_lista = concat( "lista" , name , "_" ) ;
			/* costruisco la lista di raggruppamento */
			new->list = create_list( *(new->orecchio_dx ), name_lista ) ;
			new->list = insert_in_list( new->list , *(new->orecchio_sx) ) ;
			new->list = insert_in_list( new->list , *(new->corno_dx) ) ;
			new->list = insert_in_list( new->list , *(new->corno_sx) ) ;
		}
		/* restituisco l' oggetto */
		return new ;
	}
}

#endif 
