#ifndef __FACCIA_E__
#define __FACCIA_E__

/* 
	MODULO PER LA GESTIONE DELLE SUPERFICI DELLA FACCIA
*/

#include "descriptor/descriptor.h"

/* percorsi dei file di superficie a partire dalla directory della libreria corrente */
char* path_faccia_sotto_naso = "s/faccia_sotto_naso_s.db" ;
char* path_guancia_dx = "s/guancia_dx_s.db" ;
char* path_guancia_sx = "s/guancia_sx_s.db" ;
char* path_soppracciglio_dx = "s/sopracciglio_dx_s.db" ;
char* path_soppracciglio_sx = "s/soppracciglio_sx_s.db" ;
char* path_tempia_dx = "s/tempia_dx_s.db" ;
char* path_tempia_sx = "s/tempia_sx_s.db" ;

/* indico la struttura contentne tutti gli ITEM */
typedef struct {
	/* singoli oggetti ITEM per le singole superfici */
	ITEM 	*faccia_sotto_naso , 
			*guancia_dx , 
			*guancia_sx , 
			*soppracciglio_dx , 
			*soppracciglio_sx , 
			*tempia_dx ,
			*tempia_sx ;
	/* oggetto ITEM che racchiude gli oggetti item delle superfici per applicare 
	le trasformazioni spaziali */
	ITEM list ;
} faccia ;

/* funzione che crea una nuova struttura faccia e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
faccia* new_faccia( char *name , char *path , ITEM *attr ) ;

/* ********************************************************************************************
IMPLEMENTAZIONE 
********************************************************************************************* */

#include "../../../common/utilities.e" /* concat */

#include <stdio.h> /* fprintf */
#include <stdlib.h> /* malloc , exit */
#include <string.h> /* strdup */
				 
/* funzione che crea una nuova struttura e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
faccia* new_faccia( char *name , char *path , ITEM *attr ) {
	/* controllo i valori di input */
	if (( name == NULL ) || ( path == NULL ) || ( attr == NULL )) {
		fprintf( stderr , "valori di input non validi\n" ) ;
		exit( EXIT_FAILURE ) ;
	} else {
		/* alloco memoria alla struttura */
		faccia* new ;
		new = malloc( sizeof ( faccia )) ;
		if ( new == NULL ){
			fprintf( stderr , "impossibile allocare ulteriore memoria\n" ) ;
			exit( EXIT_FAILURE ) ;
		}
		/* ricavo le superfici */
		new->faccia_sotto_naso = get_surface( path_faccia_sotto_naso , path ) ;
		new->guancia_dx = get_surface( path_guancia_dx , path ) ;
		new->guancia_sx = get_surface( path_guancia_sx , path ) ;
		new->soppracciglio_dx = get_surface( path_soppracciglio_dx , path ) ;
		new->soppracciglio_sx = get_surface( path_soppracciglio_sx , path ) ;
		new->tempia_dx = get_surface( path_tempia_dx , path ) ;
		new->tempia_sx = get_surface( path_tempia_sx , path ) ;
		/* collego le superfici agli attributi */
		set_attribute( *(new->faccia_sotto_naso) , *attr);
		set_attribute( *(new->guancia_dx), *attr);
		set_attribute( *(new->guancia_sx), *attr);
		set_attribute( *(new->soppracciglio_dx), *attr);
		set_attribute( *(new->soppracciglio_sx), *attr);
		set_attribute( *(new->tempia_dx), *attr);
		set_attribute( *(new->tempia_sx), *attr);
		{
			/* costruisco il nome da dare alla lista */
			char* name_lista ;
			name_lista = concat( "lista" , name , "_" ) ;
			/* costruisco la lista di raggruppamento */
			new->list = create_list( *(new->faccia_sotto_naso ), name_lista ) ;
			new->list = insert_in_list( new->list , *(new->guancia_dx) ) ;
			new->list = insert_in_list( new->list , *(new->guancia_sx) ) ;
			new->list = insert_in_list( new->list , *(new->soppracciglio_dx) ) ;
			new->list = insert_in_list( new->list , *(new->soppracciglio_sx) ) ;
			new->list = insert_in_list( new->list , *(new->tempia_dx) ) ;
			new->list = insert_in_list( new->list , *(new->tempia_sx) ) ;
		}
		/* restituisco l' oggetto */
		return new ;
	}
}

#endif 
