#ifndef __JET_PILDER_E__
#define __JET_PILDER_E__

/* 
	MODULO PER LA GESTIONE DELLE SUPERFICI DEL JET PILDER
*/

#include "descriptor/descriptor.h"

/* percorsi dei file di superficie a partire dalla directory della libreria corrente */
char* path_base_laterale_dx = "s/base_laterale_dx_s.db" ;
char* path_base_laterale_sx = "s/base_laterale_sx_s.db" ;
char* path_base_sup = "s/base_sup_s.db" ;
char* path_cupola = "s/cupola_s.db" ;
char* path_cerchietto = "s/cerchietto_s.db" ;
char* path_coda = "s/coda_s.db" ;
char* path_sopra_cerchietto = "s/sopra_cerchietto_s.db" ;
char* path_aletta_dx = "s/aletta_dx_s.db" ;
char* path_aletta_sx = "s/aletta_sx_s.db" ;
char* path_spessore_aletta = "s/spessore_aletta_s.db" ;

/* indico la struttura contentne tutti gli ITEM */
typedef struct {
	/* singoli oggetti ITEM per le singole superfici */
	ITEM 	*base_laterale_dx , 
 			*base_laterale_sx ,
			*base_sup ,
   			*cupola ,
	  		*cerchietto , 
	 		*coda ,
			*sopra_cerchietto ,
			*aletta_dx ,
			*aletta_sx ,
			*spessore_aletta ;
		/* oggetto ITEM che racchiude gli oggetti item delle superfici per applicare 
	le trasformazioni spaziali */
	ITEM list ;
} jet_pilder ;

/* funzione che crea una nuova struttura e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
jet_pilder* new_jet_pilder( char *name , char *path , ITEM *scocca_attr , ITEM *parabrezza_attr ) ;

/* ********************************************************************************************
IMPLEMENTAZIONE 
********************************************************************************************* */

#include "../../../common/utilities.e" /* concat */

#include <stdio.h> /* fprintf */
#include <stdlib.h> /* malloc , exit */
#include <string.h> /* strdup */

/* funzione che crea una nuova struttura e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
jet_pilder* new_jet_pilder( char *name , char *path , ITEM *scocca_attr , ITEM *parabrezza_attr ) {
	/* controllo i valori di input */
	if (( name == NULL ) || ( path == NULL ) || ( scocca_attr == NULL )|| ( parabrezza_attr == NULL )) {
		fprintf( stderr , "valori di input non validi\n" ) ;
		exit( EXIT_FAILURE ) ;
	} else {
		/* alloco memoria alla struttura */
		jet_pilder* new ;
		new = malloc( sizeof ( jet_pilder )) ;
		if ( new == NULL ){
			fprintf( stderr , "impossibile allocare ulteriore memoria\n" ) ;
			exit( EXIT_FAILURE ) ;
		}
		/* ricavo le superfici */
		new->base_laterale_dx = get_surface( path_base_laterale_dx , path ) ;
		new->base_laterale_sx = get_surface( path_base_laterale_sx , path ) ;
		new->base_sup = get_surface( path_base_sup , path ) ;
		new->cupola = get_surface( path_cupola , path ) ;
		new->cerchietto = get_surface( path_cerchietto , path ) ;
		new->coda = get_surface( path_coda , path ) ;
		new->sopra_cerchietto = get_surface( path_sopra_cerchietto , path ) ;
		new->aletta_dx = get_surface( path_aletta_dx , path ) ;
		new->aletta_sx = get_surface( path_aletta_sx , path ) ;
		new->spessore_aletta = get_surface( path_spessore_aletta , path ) ;
		/* collego le superfici agli attributi */
		set_attribute( *(new->base_laterale_dx) , *scocca_attr);
		set_attribute( *(new->base_laterale_sx), *scocca_attr);
		set_attribute( *(new->base_sup), *scocca_attr);
		set_attribute( *(new->cupola), *parabrezza_attr);
		set_attribute( *(new->cerchietto), *scocca_attr);
		set_attribute( *(new->coda), *scocca_attr);
		set_attribute( *(new->sopra_cerchietto), *scocca_attr);
		set_attribute( *(new->aletta_dx), *scocca_attr);
		set_attribute( *(new->aletta_sx), *scocca_attr);
		set_attribute( *(new->spessore_aletta), *scocca_attr);
		{
			/* costruisco il nome da dare alla lista */
			char* name_lista ;
			name_lista = concat( "lista" , name , "_" ) ;
			/* costruisco la lista di raggruppamento */
			new->list = create_list( *(new->base_laterale_dx ), name_lista ) ;
			new->list = insert_in_list( new->list , *(new->base_laterale_sx) ) ;
			new->list = insert_in_list( new->list , *(new->base_sup) ) ;
			new->list = insert_in_list( new->list , *(new->cupola) ) ;
			new->list = insert_in_list( new->list , *(new->cerchietto) ) ;
			new->list = insert_in_list( new->list , *(new->coda) ) ;
			new->list = insert_in_list( new->list , *(new->sopra_cerchietto) ) ;
			new->list = insert_in_list( new->list , *(new->aletta_dx) ) ;
			new->list = insert_in_list( new->list , *(new->aletta_sx) ) ;
			new->list = insert_in_list( new->list , *(new->spessore_aletta) ) ;
		}
		/* restituisco l' oggetto */
		return new ;
	}
}

#endif 
