#ifndef __NASO_E__
#define __NASO_E__

/* 
	MODULO PER LA GESTIONE DELLE SUPERFICI DEL NASO
*/

#include "descriptor/descriptor.h"

/* percorsi dei file di superficie a partire dalla directory della libreria corrente */
char* path_parete_naso_punta = "s/parete_naso_punta_s.db" ;
char* path_parete_naso_down = "s/parete_naso_down_s.db" ;
char* path_parete_naso_sx = "s/parete_naso_sx_s.db" ;
char* path_parete_naso_dx = "s/parete_naso_dx_s.db" ;
char* path_parete_naso_up = "s/parete_naso_up_s.db" ;

/* indico la struttura contentne tutti gli ITEM */
typedef struct {
	/* singoli oggetti ITEM per le singole superfici */
	ITEM 	*parete_naso_punta , 
			*parete_naso_down , 
			*parete_naso_sx , 
			*parete_naso_dx , 
			*parete_naso_up ;
		/* oggetto ITEM che racchiude gli oggetti item delle superfici per applicare 
	le trasformazioni spaziali */
	ITEM list ;
} naso ;

/* funzione che crea una nuova struttura aeratore e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
naso* new_naso( char *name , char *path , ITEM *attr ) ;

/* ********************************************************************************************
IMPLEMENTAZIONE 
********************************************************************************************* */

#include "../../../common/utilities.e" /* concat */

#include <stdio.h> /* fprintf */
#include <stdlib.h> /* malloc , exit */
#include <string.h> /* strdup */

/* funzione che crea una nuova struttura aeratore e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
naso* new_naso( char *name , char *path , ITEM *attr ) {
	/* controllo i valori di input */
	if (( name == NULL ) || ( path == NULL ) || ( attr == NULL )) {
		fprintf( stderr , "valori di input non validi\n" ) ;
		exit( EXIT_FAILURE ) ;
	} else {
		/* alloco memoria alla struttura */
		naso* new ;
		new = malloc( sizeof ( naso )) ;
		if ( new == NULL ){
			fprintf( stderr , "impossibile allocare ulteriore memoria\n" ) ;
			exit( EXIT_FAILURE ) ;
		}
		/* ricavo le superfici */
		new->parete_naso_punta = get_surface( path_parete_naso_punta , path ) ;
		new->parete_naso_down = get_surface( path_parete_naso_down , path ) ;
		new->parete_naso_sx = get_surface( path_parete_naso_sx , path ) ;
		new->parete_naso_dx = get_surface( path_parete_naso_dx , path ) ;
		new->parete_naso_up = get_surface( path_parete_naso_up , path ) ;
		/* collego le superfici agli attributi */
		set_attribute( *(new->parete_naso_punta) , *attr);
		set_attribute( *(new->parete_naso_down), *attr);
		set_attribute( *(new->parete_naso_sx), *attr);
		set_attribute( *(new->parete_naso_dx), *attr);
		set_attribute( *(new->parete_naso_up), *attr);
		{
			/* costruisco il nome da dare alla lista */
			char* name_lista ;
			name_lista = concat( "lista" , name , "_" ) ;
			/* costruisco la lista di raggruppamento */
			new->list = create_list( *(new->parete_naso_punta ), name_lista ) ;
			new->list = insert_in_list( new->list , *(new->parete_naso_down) ) ;
			new->list = insert_in_list( new->list , *(new->parete_naso_sx) ) ;
			new->list = insert_in_list( new->list , *(new->parete_naso_dx) ) ;
			new->list = insert_in_list( new->list , *(new->parete_naso_up) ) ;
		}
		/* restituisco l' oggetto */
		return new ;
	}
}

#endif 
