#ifndef __TESTA_E__
#define __TESTA_E__

/* 
	MODULO PER LA GESTIONE DELLE SUPERFICI DELLA TESTA
*/

#include "descriptor/descriptor.h"

/* sottostrutture */
#include "aeratore_centrale/aeratore_centrale.e"
#include "aeratore_laterale/aeratore_laterale.e"
#include "bocca/bocca.e"
#include "collo/collo.e"
#include "coppa/coppa.e"
#include "corna_orecchie/corna_orecchie.e"
#include "corona_fronte/corona_fronte.e"
#include "faccia/faccia.e"
#include "naso/naso.e"
#include "occhi/occhi.e"
#include "interno_corona/interno_corona.e"
#include "incavo_jet_pilder/incavo_jet_pilder.e"
#include "jet_pilder/jet_pilder.e"

/* indico la struttura contentne tutti gli ITEM */
typedef struct {
	/* sottostrutture */
	aeratore_centrale* aer_centr ;
	aeratore_laterale* aer_lat ;
	bocca* boc ;
	collo* col ;
	coppa* cop ;
	corna_orecchie* c_o ;
	corona_fronte* c_f ;
	faccia* fac ;
	naso* nas ;
	occhi* occ ;
	interno_corona* i_c ;
	incavo_jet_pilder* inc ;
	jet_pilder* jet ;
	/* oggetto ITEM che racchiude gli oggetti item delle superfici 
	delle sottostrutture per applicare le trasformazioni spaziali */
	ITEM list ;
} testa ;

/* funzione che crea una nuova struttura testa e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
testa* new_testa( char *name , char *path , ITEM *occhi_a , ITEM *corna_a , ITEM *viso_a , ITEM *maschera_a , ITEM *saldature_a , ITEM *interno_aeratori_a , ITEM *jet_pilder_a , ITEM *parabrezza_jet_a ) ;

/* ********************************************************************************************
IMPLEMENTAZIONE 
********************************************************************************************* */

#include "../../common/utilities.e" /* concat */

#include <stdio.h> /* fprintf */
#include <stdlib.h> /* malloc , exit */
#include <string.h> /* strdup */

/* funzione che crea una nuova struttura testa e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
testa* new_testa( char *name , char *path , ITEM *occhi_a , ITEM *corna_a , ITEM *viso_a , ITEM *maschera_a , ITEM *saldature_a , ITEM *interno_aeratori_a , ITEM *jet_pilder_a , ITEM *parabrezza_jet_a ) {
	/* controllo i valori di input */
	if (( name == NULL ) || ( path == NULL ) || ( occhi_a == NULL ) || 
		( corna_a == NULL ) || ( viso_a == NULL ) || ( maschera_a == NULL ) || 
		( saldature_a == NULL ) || ( interno_aeratori_a == NULL ) || ( parabrezza_jet_a == NULL ) ||
		( jet_pilder_a == NULL ) || ( interno_aeratori_a == NULL )|| ( interno_aeratori_a == NULL )) {
		fprintf( stderr , "valori di input non validi\n" ) ;
		exit( EXIT_FAILURE ) ;
	} else {
		/* alloco memoria alla struttura */
		testa* new ;
		new = malloc( sizeof ( testa )) ;
		if ( new == NULL ){
			fprintf( stderr , "impossibile allocare ulteriore memoria\n" ) ;
			exit( EXIT_FAILURE ) ;
		}
		{
			char *path_aer_centr , *path_aer_lat , *path_boc , *path_col , *path_cop , *path_jet ,
   				*path_c_o , *path_c_f , *path_fac , *path_nas , *path_occ , *path_i_c , *path_inc ;
			/* ricavo gli indirizzi delle sottostrutture */
			path_aer_centr = concat_dir( path , "aeratore_centrale" ) ;
			path_aer_lat = concat_dir( path , "aeratore_laterale" ) ;
			path_boc = concat_dir( path , "bocca" ) ;
			path_col = concat_dir( path , "collo" ) ;
			path_cop = concat_dir( path , "coppa" ) ;
			path_c_o = concat_dir( path , "corna_orecchie" ) ;
			path_c_f = concat_dir( path , "corona_fronte" ) ;
			path_fac = concat_dir( path , "faccia" ) ;
			path_nas = concat_dir( path , "naso" ) ;
			path_occ = concat_dir( path , "occhi" ) ;
			path_i_c = concat_dir( path , "interno_corona" ) ;
			path_inc = concat_dir( path , "incavo_jet_pilder" ) ;
			path_jet = concat_dir( path , "jet_pilder" ) ;
			/* ricavo le sottostrutture */
			new->aer_centr = new_aeratore_centrale( "aer_centr" , path_aer_centr , interno_aeratori_a , maschera_a ) ;
			new->aer_lat = new_aeratore_laterale( "aer_lat" , path_aer_lat , interno_aeratori_a , maschera_a ) ;
			new->boc = new_bocca( "bocca" , path_boc , maschera_a ) ;
			new->col = new_collo( "collo" , path_col , maschera_a ) ;
			new->cop = new_coppa( "coppa" , path_cop , maschera_a ) ;
			new->c_o = new_corna_orecchie( "corna_orecchie" , path_c_o , corna_a , maschera_a ) ;
			new->c_f = new_corona_fronte( "corona_fronte" , path_c_f , maschera_a ) ;
			new->fac = new_faccia( "faccia" , path_fac , viso_a ) ;
			new->nas = new_naso( "naso" , path_nas , viso_a ) ;
			new->occ = new_occhi( "occhi" , path_occ , occhi_a , saldature_a ) ;
			new->i_c = new_interno_corona( "interno_corona" , path_i_c , maschera_a ) ;
			new->inc = new_incavo_jet_pilder( "incavo_jet_pilder" , path_inc , viso_a ) ;
			new->jet = new_jet_pilder( "jet_pilder" , path_jet , jet_pilder_a , parabrezza_jet_a ) ;
			/* libero la memoria delle stringhe */
			free(path_aer_centr ) ; free(path_aer_lat ) ; free(path_jet ) ;
			free(path_boc ) ; free(path_col ) ; free(path_cop ) ;
			free(path_c_o ) ; free(path_c_f ) ; free(path_fac ) ;
			free(path_nas ) ; free(path_occ ) ; free(path_i_c ) ; free(path_inc ) ;
		}
		{
			/* costruisco il nome da dare alla lista */
			char* name_lista ;
			name_lista = concat( "lista" , name , "_" ) ;
			/* costruisco la lista di raggruppamento */
			new->list = create_list( new->aer_centr->list , name_lista ) ;
			new->list = insert_in_list( new->list , new->aer_lat->list ) ;
			new->list = insert_in_list( new->list , new->boc->list ) ;
			new->list = insert_in_list( new->list , new->col->list ) ;
			new->list = insert_in_list( new->list , new->cop->list ) ;
			new->list = insert_in_list( new->list , new->c_o->list ) ;
			new->list = insert_in_list( new->list , new->c_f->list ) ;
			new->list = insert_in_list( new->list , new->fac->list ) ;
			new->list = insert_in_list( new->list , new->nas->list ) ;
			new->list = insert_in_list( new->list , new->occ->list ) ;
			new->list = insert_in_list( new->list , new->i_c->list ) ;
			new->list = insert_in_list( new->list , new->inc->list ) ;
			new->list = insert_in_list( new->list , new->jet->list ) ;
		}
		/* restituisco l' oggetto */
		return new ;
	}
}

#endif 
