#ifndef __SFONDO_E__
#define __SFONDO_E__

/* 
	MODULO PER LA GESTIONE DELLE SUPERFICI DELLO SFONDO
*/

#include "descriptor/descriptor.h"

/* percorsi dei file di superficie a partire dalla directory della libreria corrente */
char* path_sfera = "s/sfera_s.db" ;
char* path_piano = "s/piano_s.db" ;
/* percorso per arrivare alle texture */
char* path_tex_cielo = "textures/cielo.hr" ;
char* path_tex_suolo = "textures/grass.hr" ;

/* indico la struttura contentne tutti gli ITEM */
typedef struct {
	/* singoli oggetti ITEM per le singole superfici */
	ITEM 	*cielo ,
 			*suolo ;
	/* attributi delle superfici */
	ITEM 	*cielo_attr ,
 			*suolo_attr ;		
		/* oggetto ITEM che racchiude gli oggetti item delle superfici per applicare 
	le trasformazioni spaziali */
	ITEM list ;
} sfondo ;

/* funzione che crea una nuova struttura e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
sfondo* new_sfondo( char *name , char *path ) ;

/* ********************************************************************************************
IMPLEMENTAZIONE 
********************************************************************************************* */

#include "../common/utilities.e" /* concat */

#include <stdio.h> /* fprintf */
#include <stdlib.h> /* malloc , exit */
#include <string.h> /* strdup */
				 
/* costante indicante la scala di ingrandimento dello sfondo */
#define SFONDO_SCALE (float)50.0

/* funzione che crea una nuova struttura e la restituisce , bisogna inserire il nome 
da dare alla struttura e il path che va dal programma effettivo alla libreria */
sfondo* new_sfondo( char *name , char *path ) {
	/* controllo i valori di input */
	if (( name == NULL ) || ( path == NULL )) {
		fprintf( stderr , "valori di input non validi\n" ) ;
		exit( EXIT_FAILURE ) ;
	} else {
		/* alloco memoria alla struttura */
		sfondo* new ;
		new = malloc( sizeof ( sfondo )) ;
		if ( new == NULL ){
			fprintf( stderr , "impossibile allocare ulteriore memoria\n" ) ;
			exit( EXIT_FAILURE ) ;
		}
		/* ricavo le superfici */
		new->cielo = get_surface( path_sfera , path ) ;
		new->suolo = get_surface( path_piano , path ) ;
		/* costruisco gli attributi di default */
		new->cielo_attr = get_attribute( name , "cielo_attr" ) ;
		new->suolo_attr = get_attribute( name , "suolo_attr" ) ;
		/* setto gli attributi */
		set_color(*(new->cielo_attr), 1.0, 1.0, 1.0, 0.3, 0.7);
		set_reflectivity(*(new->cielo_attr), 0.0, 2.0);
		set_color(*(new->suolo_attr), 1.0, 1.0, 1.0, 0.3, 0.7);
		/* collego le superfici agli attributi */
		set_attribute( *(new->cielo), *(new->cielo_attr));
		set_attribute( *(new->suolo), *(new->suolo_attr));
		/* indico la scala di trasformazione */
		set_scale(*(new->cielo), SFONDO_SCALE, SFONDO_SCALE, SFONDO_SCALE);
		set_scale(*(new->suolo), SFONDO_SCALE, SFONDO_SCALE, SFONDO_SCALE);
		{
			/* trovo i percorsi per le texture */
			char *cielo_path , *suolo_path ;
			cielo_path = concat_dir( path , path_tex_cielo ) ;
			suolo_path = concat_dir( path , path_tex_suolo ) ;
			/* collego le texture */
			set_domain_texture(*(new->cielo), *(new->cielo_attr), cielo_path ,
							   0.0, 0.2, 0.0, 0.2, 1, 0.3, 0.7);
			set_domain_texture(*(new->suolo), *(new->suolo_attr), suolo_path,
							   0.0, 0.2, 0.0, 0.2, 1, 0.3, 0.7);
			free( cielo_path ) ;
			free( suolo_path ) ;
		}
		{
			/* costruisco il nome da dare alla lista */
			char* name_lista ;
			name_lista = concat( "lista" , name , "_" ) ;
			/* costruisco la lista di raggruppamento */
			new->list = create_list( *(new->cielo), name_lista ) ;
			new->list = insert_in_list( new->list , *(new->suolo) ) ;
		}
		/* restituisco l' oggetto */
		return new ;
	}
}

#endif 

