/* primitives.c */

#ifndef DEFINEH
#include "globals/define.h"
#endif

#include <string.h>
#include "hrayt/vec_macros.h"
#include "hrayt/types.h"

#include "matrix/matrix.h"

#include "descriptor/types.h"
#include "descriptor/services.h"
#include "descriptor/globals.h"
#include "descriptor/proto/primitives.h"

ITEM create_nurbs(int order_u, int order_v,
		  int num_vertices_u, int num_vertices_v,
		  VEC_vector_t **vertices, VEC_real_t**coefficients,
		  VEC_real_t *knots_u, VEC_real_t *knots_v,
		  BOOLEAN is_surface, BOOLEAN normal_inside,
		  Trimming_tree_t trimming_tree, char *name) {

  ITEM 		it;
  Thing_t 	*th;
  Nurbs_t 	*b;
  int 		i,j;

  routine = "create_nurbs";
  if (order_u > 24 || order_v > 24) 
    descript_error(DEDOMAIN,"order must be < 24");
  th = create_primitive(NURBS);
  b = (Nurbs_t *)descript_malloc(sizeof(Nurbs_t),1);
  strcpy(th->name, name);
  th->nurbs.i = num_nurbs;
  nurbs[num_nurbs++] = b;	

  b->num_vertices_u = num_vertices_u;
  b->num_vertices_v = num_vertices_v;
  b->order_u = order_u;
  b->order_v = order_v;
  b->num_knots_u = order_u + num_vertices_u ;
  b->num_knots_v = order_v + num_vertices_v ;

  allocate_nurbs_arrays(b);

  for (i=0; i<b->num_knots_u ; i++)
    b->knots_u[i]=knots_u[i];

  for (i=0; i<b->num_knots_v ; i++)
    b->knots_v[i]=knots_v[i];

  for (i = 0; i < num_vertices_u; i++)
    for (j = 0; j < num_vertices_v; j++) {
      VEC_copy(b->vertices[i][j], vertices[i][j]);
      b->coefficients[i][j]=coefficients[i][j];
    }
  b->is_surface=is_surface;
  b->normal_inside=normal_inside;
  b->trimming_tree=trimming_tree;
  nurbs_extent(&(b->extent),b);
  INIT_ITEM(it,THING,tp,th);
  return(it);
}

ITEM read_nurbs(char *filename) {

  ITEM    it;
  FILE    *fp;
  Thing_t *th;
  Nurbs_t *b;
  int     u,v;
  char    s[255];
  int     cnt;

  routine = "read_nurbs";
  if ((fp = fopen(filename, "r")) == NULL) 
    descript_error(DEIO,"cannot open file");

  th = create_primitive(NURBS);
  b = (Nurbs_t *)descript_malloc(sizeof(Nurbs_t),1);
  th->nurbs.i = num_nurbs;
  /* modifica */
  strcpy(th->name, filename);
  nurbs[num_nurbs++] = b;	

  nextline(fp, s);
  if ( sscanf(s, "FILENAME : %s", b->name ) != 1)
    descript_error(DEIO,"error reading surface name");
  /* a b viene assegnato il nome interno al file , quindi è locale alla directory , mentre 
  potrebbe prendere il nome interno al file e non dall' indirizzo passato , che un indirizzo 
  completo . faccio una strcopy anche a b->name dopo aver controlato che il file esista */
  strcpy(b->name, filename);
  /* fine della modifica */
/*printf("FILENAME : %s\n", b->name );*/
  nextline(fp, s);
/*printf("%s\n",s);*/
  nextline(fp, s);
  if (sscanf(s, "%d%d", &(b->order_u), &(b->order_v)) != 2)
    descript_error(DEIO,"error reading degree");
/*printf("%d %d\n", b->order_u, b->order_v);*/

  b->order_u++;
  b->order_v++; 

  nextline(fp, s);
/*printf("%s\n",s);*/
  nextline(fp, s);
  if (sscanf(s, "%d%d", &(b->num_vertices_u), &(b->num_vertices_v)) != 2)
    descript_error(DEIO,"error reading number of vertices");
/*printf("%d %d\n", b->num_vertices_u, b->num_vertices_v);*/
  nextline(fp, s);
/*printf("%s\n",s);*/
  nextline(fp, s);
  if (sscanf(s, "%d%d", &(b->num_knots_u), &(b->num_knots_v)) != 2)
    descript_error(DEIO,"error reading number of knots");
/*printf("%d %d\n",b->num_knots_u,b->num_knots_v);*/
  if(b->num_knots_u != b->order_u + b->num_vertices_u || 
     b->num_knots_v != b->order_v + b->num_vertices_v)
    descript_error(DECONSTR,"invalid number of knots");

  allocate_nurbs_arrays(b);

  nextline(fp, s);
/*printf("%s\n",s);*/
  for (u = 0; u < b->num_vertices_u; u++) 
    for (v = 0; v < b->num_vertices_v; v++) {
      nextline(fp, s);
      cnt = sscanf(s, "%lf%lf%lf%lf",
		   &(b->vertices[u][v][0]), &(b->vertices[u][v][1]), 
		   &(b->vertices[u][v][2]),&(b->coefficients[u][v]));
/*    printf("%f %f %f %f\n", b->vertices[u][v][0], b->vertices[u][v][1], b->vertices[u][v][2],b->coefficients[u][v]);*/
      if (cnt < 4) 
	descript_error(DEIO,"error reading vertices");
  }

  nextline(fp, s);
/*printf("%s\n",s);*/  
  for(u = 0; u < b->num_knots_u; u++) {
    nextline(fp, s);
    cnt = sscanf(s, "%lf",&(b->knots_u[u]));
    if (cnt < 1) 
      descript_error(DEIO,"error reading knots in u parameter");
/*    printf("%f\n",b->knots_u[u]);*/
  }

  nextline(fp, s);
/*printf("%s\n",s);*/
  for (v = 0; v < b->num_knots_v; v++) {
    nextline(fp, s);
    cnt = sscanf(s, "%lf",&(b->knots_v[v]));
/*    printf("%f\n",b->knots_v[v]);*/
    if (cnt < 1) 
      descript_error(DEIO,"error reading knots in v parameter");			
  }

  nextline(fp, s); 
  if (sscanf(s, "%d", &(b->is_surface)) != 1) {
    b->is_surface = TRUE;
    b->normal_inside = FALSE;
    b->trimming_tree.trim_outside = FALSE;
    b->trimming_tree.num_curves = 0;
    b->trimming_tree.curves = (Trimming_curve_t *)0;
  }
  else {	
    nextline(fp, s);
    if (sscanf(s, "%d", &(b->normal_inside)) != 1)
      descript_error(DEIO,"error reading normal inside/outside flag");			

    /* lettura albero curve di trimming */

    nextline(fp, s);
    if (sscanf(s, "%d", &(b->trimming_tree.trim_outside)) != 1)
      descript_error(DEIO,"error reading trimming inside/outside flag");			

    nextline(fp, s);
    if (sscanf(s, "%d", &(b->trimming_tree.num_curves)) != 1)
      descript_error(DEIO,"error reading number of trimming curves");			
	
    read_trimming_tree(fp, &(b->trimming_tree));
  }
  fclose(fp);
  nurbs_extent(&(b->extent),b);
  INIT_ITEM(it,THING,tp,th);
  return(it);
}

ITEM read_obj(char *filename, char *name) {
  FILE *fp;
  ITEM it;
  char s[255],
       surf_fname[255];
  int  i,surf_no;
  
  if ((fp = fopen(filename, "r")) == NULL) 
    descript_error(DEIO,"cannot open file");
  nextline(fp, s);
  if ( sscanf(s, "FILENAME : %s", s ) != 1)
    descript_error(DEIO,"error reading objname");
  nextline(fp, s);
  if (sscanf(s, "%d", &surf_no) != 1)
    descript_error(DEIO,"error reading number of surfaces");
  
  for (i=0; i<surf_no; i++) {
    nextline(fp, s);
    if ( sscanf(s, "%s", surf_fname ) != 1)
      descript_error(DEIO,"error reading surface name");
    if(!i)
      it = create_list(read_nurbs(surf_fname), name);
    else
      it = insert_in_list(it, read_nurbs(surf_fname));
  }
  
  fclose(fp);
  return(it);
}

static	void allocate_nurbs_arrays(Nurbs_t *b) {

  int	i;

  b->vertices = (VEC_vector_t **)
    descript_malloc(sizeof(VEC_vector_t *),(size_t)b->num_vertices_u);
  b->coefficients = (VEC_real_t **)
    descript_malloc(sizeof(VEC_real_t *),(size_t)b->num_vertices_u);
  for (i = 0; i < b->num_vertices_u; i++) {
    b->vertices[i] = (VEC_vector_t *)
      descript_malloc(sizeof(VEC_vector_t),(size_t)b->num_vertices_v);
    b->coefficients[i] = (VEC_real_t *)
      descript_malloc(sizeof(VEC_real_t),(size_t)b->num_vertices_v);
  }
  b->knots_u = (VEC_real_t *)
    descript_malloc(sizeof(VEC_real_t),
		 (size_t)(b->order_u+b->num_vertices_u));
  b->knots_v = (VEC_real_t *)
    descript_malloc(sizeof(VEC_real_t),
		 (size_t)(b->order_v+b->num_vertices_v));
}

static Thing_t *create_primitive(int shape) {

  Thing_t *th;
  unsigned int j;

  th = (Thing_t *)descript_malloc(sizeof(Thing_t),1);
  th->geo_shape = shape;
  th->transform.form = IDENTITY;
  MAT_identity(th->transform.Mat);
  th->texture_ix = 0;
  th->world_point_fg = FALSE;
  th->texture_matrix_form = IDENTITY;
  MAT_identity(th->texture_matrix);
  th->td.size = 0;
  th->td.type = NOTHING;
  for (j = 0; j < MAX_OBJ_ATTR; j++)
    th->attributes[j].i = 0;
  return(th);
}

static void read_trimming_tree(FILE *fp,
			  Trimming_tree_t *t) {

  int i;

  if(t->num_curves==0)
    t->curves= NULL;
  else {
    t->curves = (Trimming_curve_t *)
      descript_malloc(sizeof(Trimming_curve_t),
		   (size_t)t->num_curves);
    for(i=0;i<t->num_curves;i++) {
      t->curves[i].tree.trim_outside=!t->trim_outside;
      read_trimming_curve(fp, &(t->curves[i]));
    }
  }
}

static void read_trimming_curve(FILE *fp,
			   Trimming_curve_t *c) {

  int		i,cnt;
  static  char	s[255];

  nextline(fp, s);
  if (sscanf(s, "%d", &(c->tree.num_curves)) != 1)
    descript_error(DEIO,"error reading number of trimming subtree");			
	
  nextline(fp, s);
  if (sscanf(s, "%d", &(c->num_vertices)) != 1)
    descript_error(DEIO,"error reading number of curve vertices");			

  if(c->num_vertices==0)
    c->vertices=(VEC_vector_2_t *)0;
  else {
    c->vertices = (VEC_vector_2_t *)
      descript_malloc(sizeof(VEC_vector_2_t),c->num_vertices);
    for(i=0;i<c->num_vertices;i++) {
      nextline(fp, s);
      /* lettura vertici (v,u) */
      cnt = sscanf(s, "%lf%lf",&(c->vertices[i][1]),
		   &(c->vertices[i][0]));
      if (cnt < 2)
	descript_error(DEIO,"error reading curve vertices");
    }
  }	
  /* TESI return(*/
  /*read_trimming_tree(fp, &(c->tree)); */
}

static void nextline(FILE *fp, char *s) {
   int c, cnt = 0;

   /* skip white space */

   while ((c = getc (fp)) == ' ' ||
	  c == '\t' || c == '\n')
      ;
   if (c == EOF)
      s[0] = '\0';
   else {
     s[cnt++] = (char) c;

     /* read the rest of the line */

     while ((c = getc (fp)) != EOF && c != '\n')
       s[cnt++] = (char) c;
     s[cnt] = '\0';
   }
}

static void nurbs_extent(Extent_t *e,
			 Nurbs_t *s) {

  register int  i,j,k;

  VEC_copy(e->min_corner,s->vertices[0][0]);
  VEC_copy(e->max_corner,s->vertices[0][0]);
  for (i=0;i<s->num_vertices_u;i++)
    for (j=0;j<s->num_vertices_v;j++)
      for(k=0;k<=2;k++)
	if(s->vertices[i][j][k]<e->min_corner[k])
	  e->min_corner[k]=s->vertices[i][j][k];
        else
	  if(s->vertices[i][j][k]>e->max_corner[k])
	    e->max_corner[k]=s->vertices[i][j][k];
//  VEC_add_constant(e->min_corner,e->min_corner,-1e-4);
//  VEC_add_constant(e->max_corner,e->max_corner,1e-4);
  if (abs(e->min_corner[0]-e->max_corner[0])<1.e-4){
            e->min_corner[0]=e->min_corner[0]-1.e-4;
            e->max_corner[0]=e->max_corner[0]+1.e-4;
  }
  if (abs(e->min_corner[1]-e->max_corner[1])<1.e-4){
            e->min_corner[1]=e->min_corner[1]-1.e-4;
            e->max_corner[1]=e->max_corner[1]+1.e-4;
  }
  if (abs(e->min_corner[2]-e->max_corner[2])<1.e-4){
            e->min_corner[2]=e->min_corner[2]-1.e-4;
            e->max_corner[2]=e->max_corner[2]+1.e-4;
  }
}


